"""Integration tests for Hetzner"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class HetznerProviderTests(TestCase, IntegrationTests):
    """TestCase for Hetzner"""
    provider_name = 'hetzner'
    domain = 'softwarekollektiv.de'

    def _filter_headers(self):
        return ['Auth-API-Token']



"""Integration tests for netcup"""
from unittest import TestCase
import pytest
from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from define_tests.TheTests
class NetcupProviderTests(TestCase, IntegrationTests):
    """TestCase for netcup."""
    provider_name = 'netcup'
    domain = 'coldfix.de'

    def _filter_post_data_parameters(self):
        # actually only param[customerid, apikey, apipassword, apisessionid],
        # but I don't think this method allows filtering nested keys...
        return ['param']

    def _test_parameters_overrides(self):
        return {'api_endpoint': 'https://ccp.netcup.net/run/webservice/servers/endpoint.php?JSON'}

    @pytest.mark.skip(reason="TTL can not be set via netcup API")
    def test_provider_when_calling_list_records_after_setting_ttl(self):
        pass



"""
This module takes care of finding information about the runtime of Lexicon:
* what are the providers installed, and available
* what is the version of Lexicon
"""
import pkgutil

import pkg_resources

from lexicon import providers


def find_providers():
    """Find all providers registered in Lexicon, and their availability"""
    providers_list = sorted({modname for (_, modname, _)
                             in pkgutil.iter_modules(providers.__path__)
                             if modname != 'base'})

    try:
        distribution = pkg_resources.get_distribution('dns-lexicon')
    except pkg_resources.DistributionNotFound:
        return {provider: True for provider in providers_list}
    else:
        return {provider: _resolve_requirements(provider, distribution)
                for provider in providers_list}


def lexicon_version():
    """Retrieve current Lexicon version"""
    try:
        return pkg_resources.get_distribution('dns-lexicon').version
    except pkg_resources.DistributionNotFound:
        return 'unknown'


def _resolve_requirements(provider, distribution):
    try:
        requirements = distribution.requires([provider])
    except pkg_resources.UnknownExtra:
        # No extra for this provider
        return True
    else:
        # Extra is defined
        try:
            for requirement in requirements:
                if hasattr(requirement, 'name'):
                    pkg_resources.get_distribution(requirement.name)
                else:
                    pkg_resources.get_distribution(requirement)
        except (pkg_resources.DistributionNotFound, pkg_resources.VersionConflict):
            # At least one extra requirement is not fulfilled
            return False

    return True



"""Integration tests for DigitalOcean"""
from unittest import TestCase

import pytest
from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from define_tests.TheTests
class DigitalOceanProviderTests(TestCase, IntegrationTests):
    """TestCase for DigitalOcean"""
    provider_name = 'digitalocean'
    domain = 'foxwoodswebsites.com'

    def _filter_headers(self):
        return ['Authorization']

    @pytest.mark.skip(reason="new test, missing recording")
    def test_provider_when_calling_update_record_should_modify_record_name_specified(self):
        return



"""Module provider for Henet"""
from __future__ import absolute_import
import logging
import re

from requests import Session
# Due to optional requirement
try:
    from bs4 import BeautifulSoup
except ImportError:
    pass

from lexicon.providers.base import Provider as BaseProvider

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['he.net']


def provider_parser(subparser):
    """Configure provider parser for Henet."""
    subparser.description = """A provider for Hurricane Electric DNS.
        NOTE: THIS DOES NOT WORK WITH 2-FACTOR AUTHENTICATION.
              YOU MUST DISABLE IT IF YOU'D LIKE TO USE THIS PROVIDER.
        """
    subparser.add_argument(
        '--auth-username',
        help='specify username for authentication'
    )
    subparser.add_argument(
        '--auth-password',
        help='specify password for authentication',
    )


class Provider(BaseProvider):
    """
        he.net provider
    """
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain = self.domain
        self.domain_id = None
        self.session = None

    def _authenticate(self):
        # Create the session GET the login page to retrieve a session cookie
        self.session = Session()
        self.session.get(
            "https://dns.he.net/"
        )

        # Hit the login page with authentication info to login the session
        login_response = self.session.post(
            "https://dns.he.net",
            data={
                "email": self._get_provider_option('auth_username') or '',
                "pass": self._get_provider_option('auth_password') or ''
            }
        )

        # Parse in the HTML, if the div containing the error message is found, error
        html = BeautifulSoup(login_response.content, "html.parser")
        if html.find("div", {"id": "dns_err"}) is not None:
            LOGGER.warning("HE login failed, check HE_USER and HE_PASS")
            return False

        # Make an authenticated GET to the DNS management page
        zones_response = self.session.get("https://dns.he.net")

        html = BeautifulSoup(zones_response.content, "html.parser")
        zone_img = html.find("img", {"name": self.domain, "alt": "delete"})

        # If the tag couldn't be found, error, otherwise, return the value of the tag
        if zone_img is None:
            LOGGER.warning("Domain %s not found in account", self.domain)
            raise AssertionError("Domain {0} not found in account".format(self.domain))

        self.domain_id = zone_img["value"]
        LOGGER.debug("HENET domain ID: %s", self.domain_id)
        return True

    # Create record. If record already exists with the same content, do nothing
    def _create_record(self, rtype, name, content):
        LOGGER.debug("Creating record for zone %s", name)
        # Pull a list of records and check for ours
        records = self._list_records(rtype=rtype, name=name, content=content)
        if len(records) >= 1:
            LOGGER.warning("Duplicate record %s %s %s, NOOP", rtype, name, content)
            return True
        data = {
            "account": "",
            "menu": "edit_zone",
            "Type": rtype,
            "hosted_dns_zoneid": self.domain_id,
            "hosted_dns_recordid": "",
            "hosted_dns_editzone": "1",
            "Priority": "",
            "Name": name,
            "Content": content,
            "TTL": "3600",
            "hosted_dns_editrecord": "Submit"
        }
        ttl = self._get_lexicon_option('ttl')
        if ttl:
            if ttl <= 0:
                data['TTL'] = "3600"
            else:
                data['TTL'] = str(ttl)
        prio = self._get_lexicon_option('priority')
        if prio:
            if prio <= 0:
                data['Priority'] = "10"
            else:
                data['Priority'] = str(prio)
        self.session.post("https://dns.he.net/index.cgi", data=data)
        # Pull a list of records and check for ours
        records = self._list_records(name=name)
        if len(records) >= 1:
            LOGGER.info("Successfully added record %s", name)
            return True
        LOGGER.info("Failed to add record %s", name)
        return False

    # List all records. Return an empty list if no records found.
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is
    # received.
    def _list_records(self, rtype=None, name=None, content=None):
        return self._list_records_internal(rtype=rtype, name=name, content=content)


    def _list_records_internal(self, rtype=None, name=None, content=None, identifier=None):
        records = []
        # Make an authenticated GET to the DNS management page
        edit_response = self.session.get(
            "https://dns.he.net/?hosted_dns_zoneid={0}&menu=edit_zone&hosted_dns_editzone".format(
                self.domain_id)
        )

        # Parse the HTML response, and list the table rows for DNS records
        html = BeautifulSoup(edit_response.content, "html.parser")

        def is_dns_tr_type(klass):
            return klass and re.compile("dns_tr").search(klass)
        records = html.findAll("tr", class_=is_dns_tr_type)

        # If the tag couldn't be found, error, otherwise, return the value of the tag
        if records is None or not records:
            LOGGER.warning("Domains not found in account")
            return records

        new_records = []
        for dns_tr in records:
            tds = dns_tr.findAll("td")
            # Process HTML in the TR children to derive each object
            rec = {}
            rec['zone_id'] = tds[0].string
            rec['id'] = tds[1].string
            rec['name'] = tds[2].string
            # the 4th entry is a comment
            type_elem = tds[3].find("span", class_='rrlabel')
            rec['type'] = type_elem.string if type_elem else None
            rec['ttl'] = tds[4].string
            if tds[5].string != '-':
                rec['priority'] = tds[5]
            rec['content'] = tds[6].string
            rec['is_dynamic'] = tds[7].string == '1'
            rec = self._clean_TXT_record(rec)
            new_records.append(rec)
        records = new_records
        if identifier:
            LOGGER.debug(
                "Filtering %d records by id: %s", len(records), identifier)
            records = [record for record in records if record['id'] == identifier]
        if rtype:
            LOGGER.debug("Filtering %d records by rtype: %s", len(records), rtype)
            records = [
                record for record in records if record['type'] == rtype]
        if name:
            LOGGER.debug("Filtering %d records by name: %s", len(records), name)
            if name.endswith('.'):
                name = name[:-1]
            records = [
                record for record in records if name in record['name']]
        if content:
            LOGGER.debug("Filtering %d records by content: %s", len(records), content.lower())
            records = [
                record for record in records if record['content'].lower() == content.lower()]
        LOGGER.debug("Final records (%d): %s", len(records), records)

        return records

    # Create or update a record.
    def _update_record(self, identifier, rtype=None, name=None, content=None):
        # Delete record if it exists
        self._delete_record(identifier, rtype, name, content)
        return self._create_record(rtype, name, content)

    # Delete an existing record.
    # If record does not exist, do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        delete_record_ids = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            delete_record_ids = [record['id'] for record in records]
        else:
            delete_record_ids.append(identifier)
        LOGGER.debug("Record IDs to delete: %s", delete_record_ids)
        for rec_id in delete_record_ids:
            # POST to the DNS management UI with form values to delete the record
            delete_response = self.session.post(
                "https://dns.he.net/index.cgi",
                data={
                    "menu": "edit_zone",
                    "hosted_dns_zoneid": self.domain_id,
                    "hosted_dns_recordid": rec_id,
                    "hosted_dns_editzone": "1",
                    "hosted_dns_delrecord": "1",
                    "hosted_dns_delconfirm": "delete"
                }
            )

            # Parse the HTML response, if the <div> tag indicating success isn't found, error
            html = BeautifulSoup(delete_response.content, "html.parser")
            if html.find("div", {"id": "dns_status"}) is None:
                LOGGER.warning("Unable to delete record %s", rec_id)
                return False
        return True

    def _request(self, action='GET', url='/', data=None, query_params=None):
        # Helper _request is not used in this provider
        pass



"""Module provider for Yandex"""
from __future__ import absolute_import
import json
import logging

import requests
from lexicon.providers.base import Provider as BaseProvider


__author__ = 'Aliaksandr Kharkevich'
__license__ = 'MIT'
__contact__ = 'https://github.com/kharkevich'

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['yandex.com']


def provider_parser(subparser):
    """Generate parser provider for Yandex"""
    subparser.add_argument(
        "--auth-token",
        help="specify PDD token (https://tech.yandex.com/domain/doc/concepts/access-docpage/)")


class Provider(BaseProvider):
    """Provider class for Yandex"""
    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.api_endpoint = 'https://pddimp.yandex.ru/api2/admin/dns'

    def _authenticate(self):
        payload = self._get('/list?domain={0}'.format(self.domain))
        if payload['success'] != "ok":
            raise Exception('No domain found')
        self.domain_id = self.domain

    def _create_record(self, rtype, name, content):
        if rtype in ('CNAME', 'MX', 'NS'):
            # make sure a the data is always a FQDN for CNAMe.
            content = content.rstrip('.') + '.'

        querystring = 'domain={0}&type={1}&subdomain={2}&content={3}'.format(
            self.domain_id, rtype, self._relative_name(name), content)
        if self._get_lexicon_option('ttl'):
            querystring += "&ttl={0}".format(self._get_lexicon_option('ttl'))

        payload = self._post('/add', {}, querystring)

        return self._check_exitcode(payload, 'create_record')

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    def _list_records(self, rtype=None, name=None, content=None):
        url = '/list?domain={0}'.format(self.domain_id)
        records = []
        payload = {}

        next_url = url
        while next_url is not None:
            payload = self._get(next_url)
            if 'links' in payload \
                    and 'pages' in payload['links'] \
                    and 'next' in payload['links']['pages']:
                next_url = payload['links']['pages']['next']
            else:
                next_url = None

            for record in payload['records']:
                processed_record = {
                    'type': record['type'],
                    'name': "{0}.{1}".format(record['subdomain'], self.domain_id),
                    'ttl': record['ttl'],
                    'content': record.get('content'),
                    'id': record['record_id']
                }
                records.append(processed_record)

        if rtype:
            records = [record for record in records if record['type'] == rtype]
        if name:
            records = [record for record in records if record['name']
                       == self._full_name(name)]
        if content:
            records = [
                record for record in records if record['content'].lower() == content.lower()]

        LOGGER.debug('list_records: %s', records)
        return records

    # Just update existing record. Domain ID (domain) and Identifier (record_id) is mandatory
    def _update_record(self, identifier, rtype=None, name=None, content=None):

        if not identifier:
            LOGGER.debug(
                'Domain ID (domain) and Identifier (record_id) '
                'is mandatory parameters for this case')
            return False

        data = ''
        if rtype:
            data += '&type={0}'.format(rtype)
        if name:
            data += '&subdomain={0}'.format(self._relative_name(name))
        if content:
            data += '&content={0}'.format(content)

        payload = self._post(
            '/edit', {}, 'domain={0}&record_id={1}'.format(self.domain_id, identifier) + data)

        return self._check_exitcode(payload, 'update_record')

    # Delete an existing record.
    # If record does not exist (I'll hope), do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        delete_record_id = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            delete_record_id = [record['id'] for record in records]
        else:
            delete_record_id.append(identifier)

        LOGGER.debug('delete_records: %s', delete_record_id)

        for record_id in delete_record_id:
            self._post(
                '/del', {}, 'domain={0}&record_id={1}'.format(self.domain_id, record_id))

        # return self._check_exitcode(payload, 'delete_record')
        return True

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        default_headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'PddToken': self._get_provider_option('auth_token')
        }

        if not url.startswith(self.api_endpoint):
            url = self.api_endpoint + url

        response = requests.request(action, url, params=query_params,
                                    data=json.dumps(data),
                                    headers=default_headers)
        # if the request fails for any reason, throw an error.
        response.raise_for_status()
        if action == 'DELETE':
            return ''
        return response.json()

    def _check_exitcode(self, payload, title):  # pylint: disable=no-self-use
        if payload['success'] == 'ok':
            LOGGER.debug('%s: %s', title, payload['success'])
            return True
        if payload['error'] == 'record_exists':
            LOGGER.debug('%s: %s', title, True)
            return True
        LOGGER.debug('%s: %s', title, payload['error'])
        return False



"""Integration tests for Gransy"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class GransyProviderTests(TestCase, IntegrationTests):
    """TestCase for Gransy"""
    provider_name = 'gransy'
    domain = 'oldium.xyz'



"""Module provider for GratisDNS"""
from __future__ import absolute_import

import logging
import requests
# Due to optional requirement
try:
    from bs4 import BeautifulSoup
except ImportError:
    pass

from lexicon.providers.base import Provider as BaseProvider

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = ['gratisdns.dk']


def provider_parser(subparser):
    """Return the parser for this provider"""
    subparser.add_argument(
        "--auth-username", help="specify email address for authentication")
    subparser.add_argument(
        "--auth-password", help="specify password for authentication")


class Provider(BaseProvider):
    """Provider class for GratisDNS"""
    # XXX: Identifiers change on updates

    def __init__(self, config):
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.api_endpoint = 'https://admin.gratisdns.com'
        self.cookies = {}

    def _authenticate(self):
        # Getting required cookie "ORGID"
        payload = {"action": "logmein",
                   "login": self._get_provider_option('auth_username'),
                   "password": self._get_provider_option('auth_password')}
        # Cannot allow redirects, as we miss the cookie then
        response = requests.post(self.api_endpoint,
                                 data=payload,
                                 allow_redirects=False)
        response.raise_for_status()

        if "ORGID" not in response.cookies:
            raise Exception("Unexpected auth response")
        self.cookies["ORGID"] = response.cookies["ORGID"]

        # Make sure domain exists
        # domain is stored in self.domain from BaseProvider
        domains = self._list_domains()
        for domain in domains:
            if domain == self.domain:
                # Domain name is the ID
                self.domain_id = domain

        if self.domain_id is None:
            raise Exception('Domain {} not found'.format(self.domain))

    def _list_domains(self):
        query_params = {'action': 'dns_primarydns'}
        response = self._get(query_params=query_params)
        html = BeautifulSoup(response.content, "html.parser")
        # NOTE: This could be more robust, by checking more of the tree
        domains = [x.contents[0] for x in html.find_all("th", scope="row")]
        LOGGER.debug('list_domains: %s', domains)

        return domains

    # List all records. Return an empty list if no records found
    # type, name and content are used to filter records.
    # If possible filter during the query, otherwise filter after response is received.
    # pylint: disable=too-many-locals
    def _list_records(self, rtype=None, name=None, content=None):
        query_params = {'action': 'dns_primary_changeDNSsetup',
                        'user_domain': self.domain_id}
        response = self._get(query_params=query_params)
        html = BeautifulSoup(response.content, "html.parser")
        # List of records
        all_records = []
        # DNS records are grouped by type
        dns_record_types = html.find_all("div", 'dns-records')
        for dns_record in dns_record_types:
            # Find the title of this group, aka record type
            record_type = dns_record.find("h2").contents[0].strip()
            # The entries themselves follow in the table below
            # Each row is a single entry
            entry_table_rows = dns_record.find('tbody').find_all('tr')
            for row in entry_table_rows:
                cols = row.find_all('td')
                # Format is; name, content, ttl, EDIT/DELETE button
                record_name = cols[0].contents[0]
                record_content = cols[1].contents[0]
                record_ttl = cols[2].contents[0]
                # We have to pull the ID from the button link
                record_id = None
                button_link = cols[3].find('a')
                if button_link:
                    start = button_link['href'].index('&id=')+4
                    end = button_link['href'].index('&', start)
                    record_id = button_link['href'][start:end]

                processed_record = {
                    'type': record_type,
                    'name': record_name,
                    'ttl': record_ttl,
                    'content': record_content,
                    'id': record_id,
                }
                all_records.append(processed_record)

        records = self._filter_records(
            records=all_records,
            rtype=rtype,
            name=name,
            content=content
        )
        LOGGER.debug('list_records: %s', records)
        return records

    # Filter a list of records based on criteria
    # NOTE: Duplicate of lexicon/providers/transip.py _filter_records
    def _filter_records(self, records, rtype=None, name=None, content=None):
        _records = []
        for record in records:
            if ((not rtype or record['type'] == rtype) and  # pylint: disable=too-many-boolean-expressions
                    (not name or self._full_name(record['name']) == self._full_name(name)) and
                    (not content or record['content'] == content)):
                _records.append(record)
        return _records

    # pylint: disable=no-self-use
    def _get_content_entry(self, rtype):
        if rtype == 'TXT':
            return 'txtdata'
        if rtype == 'A':
            return 'ip'
        if rtype == 'AAAA':
            return 'ip'
        if rtype == 'CNAME':
            return 'cname'
        LOGGER.error('unknown record type')
        return None

    # Create record. If record already exists with the same content, do nothing
    def _create_record(self, rtype, name, content):
        # Figure out TTL
        ttl = 43200
        if self._get_lexicon_option('ttl'):
            ttl = self._get_lexicon_option('ttl')

        content_entry = self._get_content_entry(rtype)
        if content_entry is None:
            return False

        # Prepare query params and payload
        query_params = {'action': 'dns_primary_record_add_' + rtype.lower(),
                        'user_domain': self.domain_id}
        payload = {'action': 'dns_primary_record_added_' + rtype.lower(),
                   'name': self._full_name(name),
                   'ttl': ttl,
                   content_entry: content,
                   'user_domain': self.domain_id}
        self._post(query_params=query_params, data=payload)
        # response = self._post(query_params=query_params, data=payload)
        # NOTE: We shouldn't return False on duplicate?
        #       The specification seems a little wage on this.
        # html = BeautifulSoup(response.content, "html.parser")
        # error = html.find('td', 'table-danger')
        # success = (error is None)

        LOGGER.debug('create_record: %s', True)
        return True

    def _lookup_record(self, identifier):
        # Pull all the records we know
        records = self._list_records()
        # Find the one with the provided identifier
        record = next((x for x in records if x['id'] == identifier), None)
        return record

    # Create or update a record.
    def _update_record(self, identifier=None, rtype=None, name=None, content=None):
        record = None
        # Try to find, either by identifier or by searching
        if identifier:
            # Find the relevant record by id
            record = self._lookup_record(identifier)
        else:
            # Find the relevant record by rtype and name
            records = self._list_records(rtype=rtype, name=name)
            if len(records) == 1:
                record = records[0]
                identifier = record['id']
        # If none is found, create it
        if record is None:
            return self._create_record(rtype, name, content)

        # Update ttl if provided
        if self._get_lexicon_option('ttl'):
            record['ttl'] = self._get_lexicon_option('ttl')
        # Update name if provided
        if name:
            record['name'] = self._full_name(name)
        if content:
            record['content'] = content

        content_entry = self._get_content_entry(rtype)
        if content_entry is None:
            return False

        # Prepare query params and payload
        query_params = {'action': 'dns_primary_record_edit_' + rtype.lower(),
                        'user_domain': self.domain_id,
                        'id': identifier}
        payload = {'action': 'dns_primary_record_update_' + rtype.lower(),
                   'id': identifier,
                   'name': record['name'],
                   'ttl': record['ttl'],
                   content_entry: record['content'],
                   'user_domain': self.domain_id}
        response = self._post(query_params=query_params, data=payload)
        html = BeautifulSoup(response.content, "html.parser")
        error = html.find('td', 'table-danger')
        success = (error is None)

        LOGGER.debug('update_record: %s', success)
        return success

    # Delete an existing record.
    # If record does not exist, do nothing.
    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        delete_record_id = []
        if not identifier:
            records = self._list_records(rtype, name, content)
            delete_record_id = [record['id'] for record in records]
        else:
            delete_record_id.append(identifier)

        LOGGER.debug('delete_records: %s', delete_record_id)

        success = True
        for record_id in delete_record_id:
            # Lookup the record to determine rtype
            if rtype is None:
                record = self._lookup_record(record_id)
                rtype = record['type']

            # Prepare query params and payload
            query_params = {'action': 'dns_primary_delete_' + rtype.lower(),
                            'user_domain': self.domain_id,
                            'id': record_id}
            # _get is intentional, get really is used for deletion on gratisdns
            response = self._get(query_params=query_params)
            html = BeautifulSoup(response.content, "html.parser")
            error = html.find('td', 'table-danger')
            success = success and (error is None)

        LOGGER.debug('delete_record: %s', success)
        return success

    # Helpers
    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}
        response = requests.request(action, self.api_endpoint + url,
                                    params=query_params,
                                    data=data,
                                    cookies=self.cookies)

        # if the request fails for any reason, throw an error.
        response.raise_for_status()
        return response



"""Provide support to Lexicon for AWS Route 53 DNS changes."""
from __future__ import absolute_import
import hashlib
import logging
import re

from lexicon.providers.base import Provider as BaseProvider


try:
    import boto3  # optional dep
    import botocore  # optional dep
except ImportError:
    pass

LOGGER = logging.getLogger(__name__)

NAMESERVER_DOMAINS = [re.compile(r'^awsdns-\d+\.\w+$')]


def provider_parser(subparser):
    """Specify arguments for AWS Route 53 Lexicon Provider."""
    subparser.add_argument("--auth-access-key",
                           help="specify ACCESS_KEY for authentication")
    subparser.add_argument("--auth-access-secret",
                           help="specify ACCESS_SECRET for authentication")
    subparser.add_argument(
        "--private-zone",
        help=("indicates what kind of hosted zone to use. If true, use "
              "only private zones. If false, use only public zones"))

    # TODO: these are only required for testing, we should figure out
    # a way to remove them & update the integration tests
    # to dynamically populate the auth credentials that are required.
    subparser.add_argument(
        "--auth-username", help="alternative way to specify the ACCESS_KEY for authentication")
    subparser.add_argument(
        "--auth-token", help="alternative way to specify the ACCESS_SECRET for authentication")


class RecordSetPaginator(object):  # pylint: disable=useless-object-inheritance
    """Paginate through complete list of record sets."""

    def __init__(self, r53_client, hosted_zone_id, max_items=None):
        """Initialize paginator."""
        self.r53_client = r53_client
        self.hosted_zone_id = hosted_zone_id
        self.max_items = max_items

    def get_record_sets(self, **kwargs):
        """Retrieve a page from API."""
        return self.r53_client.list_resource_record_sets(**kwargs)

    def get_base_kwargs(self):
        """Get base kwargs for API call."""
        kwargs = {
            'HostedZoneId': self.hosted_zone_id
        }
        if self.max_items is not None:
            kwargs.update({
                'MaxItems': str(self.max_items)
            })
        return kwargs

    def all_record_sets(self):
        """Generator to loop through current record set.

        Call next page if it exists.
        """
        is_truncated = True
        start_record_name = None
        start_record_type = None
        kwargs = self.get_base_kwargs()
        while is_truncated:
            if start_record_name is not None:
                kwargs.update({
                    'StartRecordName': start_record_name,
                    'StartRecordType': start_record_type
                })
            result = self.get_record_sets(**kwargs)
            for record_set in result.get('ResourceRecordSets', []):
                yield record_set

            is_truncated = result.get('IsTruncated', False)

            start_record_name = result.get('NextRecordName', None)
            start_record_type = result.get('NextRecordType', None)


class Provider(BaseProvider):
    """Provide AWS Route 53 implementation of Lexicon Provider interface."""

    def __init__(self, config):
        """Initialize AWS Route 53 DNS provider."""
        super(Provider, self).__init__(config)
        self.domain_id = None
        self.private_zone = self._get_provider_option('private_zone')
        # instantiate the client
        self.r53_client = boto3.client(
            'route53',
            aws_access_key_id=self._get_provider_option(
                'auth_access_key') or self._get_provider_option('auth_username'),
            aws_secret_access_key=self._get_provider_option(
                'auth_access_secret') or self._get_provider_option('auth_token')
        )

    def filter_zone(self, data):
        """Check if a zone is private"""
        if self.private_zone is not None:
            if data['Config']['PrivateZone'] != self.str2bool(self.private_zone):
                return False

        if data['Name'] != '{0}.'.format(self.domain):
            return False

        return True

    @staticmethod
    def str2bool(input_string):
        """Convert a string to boolean"""
        return input_string.lower() in ('true', 'yes')

    def _authenticate(self):
        """Determine the hosted zone id for the domain."""
        try:
            hosted_zones = self.r53_client.list_hosted_zones_by_name()[
                'HostedZones'
            ]
            hosted_zone = next(
                hz for hz in hosted_zones
                if self.filter_zone(hz)
            )
            self.domain_id = hosted_zone['Id']
        except StopIteration:
            raise Exception('No domain found')

    def _change_record_sets(self, action, rtype, name, content):
        ttl = self._get_lexicon_option('ttl')
        resource_records = []
        if isinstance(content, list):
            for i in content:
                value = '"{0}"'.format(i) if rtype in ['TXT', 'SPF'] else i
                resource_records.append({'Value': value})
        else:
            value = '"{0}"'.format(content) if rtype in ['TXT', 'SPF'] else content
            resource_records.append({'Value': value})
        try:
            self.r53_client.change_resource_record_sets(
                HostedZoneId=self.domain_id,
                ChangeBatch={
                    'Comment': '{0} using lexicon Route 53 provider'.format(
                        action
                    ),
                    'Changes': [
                        {
                            'Action': action,
                            'ResourceRecordSet': {
                                'Name': self._fqdn_name(name),
                                'Type': rtype,
                                'TTL': ttl if ttl is not None else 300,
                                'ResourceRecords': resource_records
                            }
                        }
                    ]
                }
            )
            return True
        except botocore.exceptions.ClientError as error:
            if "Duplicate Resource Record" in error.response['Error']['Message']:
                # Duplicate resource, that have been a noop. This is expected.
                return True
            LOGGER.error(str(error), exc_info=True)
            return False

    def _create_record(self, rtype, name, content):
        """Create a record in the hosted zone."""
        existing_records = self._list_record_sets(rtype, name)
        if existing_records:
            existing_record = existing_records[0]
            if isinstance(existing_records[0]['content'], list):
                return self._change_record_sets(
                    'UPSERT', existing_record['type'], existing_record['name'],
                    existing_record['content'] + [content])
            return self._change_record_sets(
                'UPSERT', rtype, name, [existing_record['content']] + [content])
        return self._change_record_sets('CREATE', rtype, name, content)

    def _update_record(self, identifier=None, rtype=None, name=None, content=None):
        """Update a record from the hosted zone."""
        if identifier:
            records = [record for record in self._list_records()
                       if identifier == _identifier(record)]
            if not records:
                raise ValueError('No record found for identifier {0}'.format(identifier))
            record = records[0]
            rtype = record['type']
            name = record['name']

        existing_records = self._list_record_sets(rtype, name)
        if not existing_records:
            raise ValueError('No matching record to update was found.')

        for existing_record in existing_records:
            if isinstance(existing_record['content'], list):
                # Multiple values in record.
                LOGGER.warning(
                    'Warning, multiple records found for given parameters, '
                    'only first entry will be updated: %s', existing_record)
                new_content = existing_record['content'].copy()
                new_content[0] = content
            else:
                new_content = content

            self._change_record_sets('UPSERT', existing_record['type'],
                                     existing_record['name'], new_content)

        return True

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        """Delete a record from the hosted zone."""
        if identifier:
            matching_records = [record for record in self._list_records()
                                if identifier == _identifier(record)]
            if not matching_records:
                raise ValueError('No record found for identifier {0}'.format(identifier))
            rtype = matching_records[0]['type']
            name = matching_records[0]['name']
            content = matching_records[0]['content']

        existing_records = self._list_record_sets(rtype, name, content)
        if not existing_records:
            raise ValueError('No record found for the provided type, name and content')

        for existing_record in existing_records:
            if isinstance(existing_record['content'], list) and content is not None:
                # multiple values in record, just remove one value and only if it actually exist
                if content in existing_record['content']:
                    existing_record['content'].remove(content)
                    self._change_record_sets('UPSERT', existing_record['type'],
                                             existing_record['name'], existing_record['content'])
            else:
                # if only one record exist, or if content is not specified, remove whole record
                self._change_record_sets('DELETE', existing_record['type'],
                                         existing_record['name'], existing_record['content'])

        return True

    def _list_records(self, rtype=None, name=None, content=None):
        """List all records for the hosted zone."""
        records = self._list_record_sets(rtype, name, content)

        flatten_records = []
        for record in records:
            if isinstance(record['content'], list):
                for one_content in record['content']:
                    flatten_record = record.copy()
                    flatten_record['content'] = one_content
                    flatten_record['id'] = _identifier(flatten_record)
                    flatten_records.append(flatten_record)
            else:
                record['id'] = _identifier(record)
                flatten_records.append(record)

        LOGGER.debug('list_records: %s', records)

        return flatten_records

    def _list_record_sets(self, rtype=None, name=None, content=None):
        records = []
        paginator = RecordSetPaginator(self.r53_client, self.domain_id)
        for record in paginator.all_record_sets():
            record_content = []
            if rtype is not None and record['Type'] != rtype:
                continue
            if name is not None and record['Name'] != self._fqdn_name(name):
                continue
            if record.get('AliasTarget', None) is not None:
                record_content = [record['AliasTarget'].get('DNSName', None)]
            if record.get('ResourceRecords', None) is not None:
                record_content = [_format_content(record['Type'], value['Value']) for value
                                  in record['ResourceRecords']]
            if content is not None and content not in record_content:
                continue

            LOGGER.debug('record: %s', record)
            records.append({
                'type': record['Type'],
                'name': self._full_name(record['Name']),
                'ttl': record.get('TTL', None),
                'content': record_content[0] if len(record_content) == 1 else record_content,
            })
        return records

    def _request(self, action='GET', url='/', data=None, query_params=None):
        # Helper _request is not used in Route53 provider
        pass


def _format_content(rtype, content):
    return content[1:-1] if rtype in ['TXT', 'SPF'] else content


def _identifier(record):
    sha256 = hashlib.sha256()
    sha256.update(('type=' + record.get('type', '') + ',').encode('utf-8'))
    sha256.update(('name=' + record.get('name', '') + ',').encode('utf-8'))
    sha256.update(('data=' + record.get('data', '') + ',').encode('utf-8'))
    return sha256.hexdigest()[0:7]



"""Integration tests for Constellix"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Constellix does not currently have a sandbox and they enforce domain
# uniqueness across the service.  You'll need your own production credentials
# and a unique domain name if you want to run these tests natively.
class ConstellixProviderTests(TestCase, IntegrationTests):
    """TestCase for Constellix"""
    provider_name = 'constellix'
    domain = 'example.org'

    def _filter_headers(self):
        return ['x-cnsdns-apiKey', 'x-cnsdns-hmac', 'x-cnsdns-requestDate']



"""Module provider for DirectAdmin hosts"""
import logging
import warnings

import requests
from requests.auth import HTTPBasicAuth

from lexicon.providers.base import Provider as BaseProvider

LOGGER = logging.getLogger(__name__)

# DirectAdmin is not tied to a specific domain, so there is nothing to specify here
NAMESERVER_DOMAINS = []

def provider_parser(subparser):
    """Return the parser for this provider"""
    subparser.add_argument(
        "--auth-password",
        help="specify password for authentication (or login key for two-factor authentication)"
    )

    subparser.add_argument(
        "--auth-username",
        help="specify username for authentication"
    )

    subparser.add_argument(
        "--endpoint",
        help="specify the DirectAdmin endpoint"
    )

# See https://www.directadmin.com/features.php?id=504 for the specification of
# the URIs for the different operations
class Provider(BaseProvider):
    """Provider class for DirectAdmin"""
    def __init__(self, config):
        super(Provider, self).__init__(config)

        self.endpoint = self._get_provider_option('endpoint')
        if self.endpoint is None:
            raise Exception('Specify endpoint of DirectAdmin')

    def _authenticate(self):
        try:
            response = self._get('/CMD_API_SHOW_DOMAINS')
        except requests.exceptions.HTTPError as err:
            # A 401 error will be returned in case of incorrect or missing
            # credentials
            cause = err.response.json()['error']
            raise Exception(cause)

        # The response is a URL encoded array containing all available domains
        domains = [domain.split('=').pop() for domain in response.split('&')]

        try:
            self.domain_id = domains.index(self.domain)
        except:
            raise Exception('Domain {0} not found'.format(self.domain))

    def _create_record(self, rtype, name, content):
        # Refuse to create duplicate records
        existing_records = self._list_records(rtype, name, content)
        if existing_records:
            return True

        query_params = {
            'action': 'add', 'json': 'yes',
            'name': '{0}.'.format(self._full_name(name)), 'type': rtype,
            'value': content
        }

        if self._get_lexicon_option('ttl'):
            query_params['ttl'] = self._get_lexicon_option('ttl')

        try:
            response = self._get('/CMD_API_DNS_CONTROL', query_params)
        except requests.exceptions.HTTPError:
            response = {'success': 'Create Failed'}

        LOGGER.debug('create_record: %s', response)

        return response['success'].lower().find('added') >= 0

    def _list_records(self, rtype=None, name=None, content=None):
        response = {'records': []}
        try:
            response = self._get('/CMD_API_DNS_CONTROL', {'json': 'yes'})
        except requests.exceptions.HTTPError as err:
            warnings.warn(err.response.text)
            raise

        records = [self._parse_response_record(record) for record in response['records']]
        if rtype:
            records = [record for record in records if record['type'] == rtype]
        if name:
            cmp_name = self._full_name(name.lower())
            records = [record for record in records if record['name'] == cmp_name]
        if content:
            records = [record for record in records if record['content'] == content]

        LOGGER.debug('list_records: %s', records)

        return records

    def _parse_response_record(self, response_record):
        # Most fields in a record response match, except for content which is
        # value
        return {
            'content': response_record['value'],
            'id': response_record['combined'],
            'name': self._full_name(response_record['name']),
            # 4 hours appears to be the default shown by DirectAdmin if no TTL
            # is set
            'ttl': response_record.get('ttl', 14400),
            'type': response_record['type']
        }

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        # Editing a record is a combination of removing the old record and
        # adding a new one, but specifying 'edit' as the action while passing
        # all parameters necessary for both deletion and creation

        if not identifier:
            # The identifier of the original record to edit is necessary to be
            # able to create the appropriate delete payload
            original_records = self.list_records(rtype, name)
            if len(original_records) > 1:
                warnings.warn('Found multiple records to edited. Cannot continue...')
                return False

            if not original_records:
                warnings.warn('Found no records to edit. Cannot continue...')
                return False

            identifier = original_records[0]['id']

        delete_key = self._determine_delete_key(identifier, rtype)

        query_params = {
            'action': 'edit', 'json': 'yes',
            delete_key: identifier,
            'name': '{0}.'.format(self._full_name(name)), 'type': rtype,
            'value': content
        }

        try:
            response = self._get('/CMD_API_DNS_CONTROL', query_params)
        except requests.exceptions.HTTPError:
            response = {'success': 'Update Failed'}

        LOGGER.debug('update_record: %s', response)

        return response['success'].lower().find('added') > 0

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        if not identifier:
            identifiers = [record['id'] for record in self._list_records(rtype, name, content)]
        else:
            identifiers = [identifier]

        response = {'success': 'noop'}
        for identifier_to_delete in identifiers:
            delete_key = self._determine_delete_key(identifier_to_delete, rtype)
            query_params = {'action': 'select', 'json': 'yes', delete_key: identifier_to_delete}

            try:
                response = self._get('/CMD_API_DNS_CONTROL', query_params)
            except requests.exceptions.HTTPError:
                response = {'success': 'Delete Failed'}

            LOGGER.debug('delete_record: %s', response)

        return response['success'].lower().find('deleted') > 0

    def _determine_delete_key(self, identifier, rtype):
        # The indicator for the record that needs to be removed is determined
        # by the type of the record and its index within all records of that
        # type. There's an additional check on the name and value which still
        # need to match for the removal to actually occur
        if not rtype:
            # An rtype is necessary to create the delete_key. However, it may
            # not be specified in which case an effort needs to be made to
            # figure it out automatically. The necessary data can be recreated
            # from the identifier
            identifier_parts = identifier.split('&')
            name = identifier_parts[0].split('=').pop()
            content = identifier_parts[1].split('=').pop()

            records = self.list_records(None, name, content)
            rtype = records[0]['type']

        existing_records = self.list_records(rtype)
        existing_record_index = 0
        for (index, record) in enumerate(existing_records):
            if record['id'] == identifier:
                existing_record_index = index

        return '{0}recs{1}'.format(rtype, existing_record_index).lower()

    def _request(self, action='GET', url='/', data=None, query_params=None):
        if data is None:
            data = {}
        if query_params is None:
            query_params = {}

        query_params['domain'] = self.domain

        response = requests.request(
            action, self.endpoint + url,
            auth=HTTPBasicAuth(
                self._get_provider_option('auth_username'),
                self._get_provider_option('auth_password')
            ),
            params=query_params
        )

        # If the request fails for any reason, throw an error
        response.raise_for_status()

        if 'json' in response.headers['Content-Type'].lower():
            return response.json()

        return response.text



"""
This unit test suite ensures that the lexicon client works correctly when used as a library.
In particular:
    - relevant provider should be resolved correctly from config,
    - config should be passed correctly to provider,
    - relevant provider method should be invoked for a given config.
"""
# pylint: disable=missing-docstring,redefined-outer-name
from __future__ import absolute_import, print_function
import importlib
import pkgutil
from types import ModuleType
import contextlib

import pytest
import mock

from lexicon.client import ProviderNotAvailableError
from lexicon.providers.base import Provider as BaseProvider


class Provider(BaseProvider):
    """
    Fake provider to simulate the provider resolution from configuration,
    and to have excution traces when lexicon client is invoked
    """
    def _authenticate(self):
        print('Authenticate action')

    def _create_record(self, rtype, name, content):
        return {'action': 'create', 'domain': self.domain,
                'type': rtype, 'name': name, 'content': content}

    def _list_records(self, rtype=None, name=None, content=None):
        return {'action': 'list', 'domain': self.domain,
                'type': rtype, 'name': name, 'content': content}

    def _update_record(self, identifier, rtype=None, name=None, content=None):
        return {'action': 'update', 'domain': self.domain, 'identifier': identifier,
                'type': rtype, 'name': name, 'content': content}

    def _delete_record(self, identifier=None, rtype=None, name=None, content=None):
        return {'action': 'delete', 'domain': self.domain, 'identifier': identifier,
                'type': rtype, 'name': name, 'content': content}

    def _request(self, action='GET', url='/', data=None, query_params=None):
        # Not use for tests
        pass


@contextlib.contextmanager
def mock_fake_provider():
    """
    Create a fake provider module, and mock relevant
    functions to make it appear as a real module.
    """
    original_iter = pkgutil.iter_modules
    original_import = importlib.import_module
    with mock.patch('lexicon.discovery.pkgutil.iter_modules') as mock_iter,\
            mock.patch('lexicon.client.importlib.import_module') as mock_import:
        def return_iter(path):
            """
            This will include an adhoc fakeprovider module
            to the normal return of pkgutil.iter_modules.
            """
            modules = list(original_iter(path))
            modules.append((None, 'fakeprovider', None))
            return modules
        mock_iter.side_effect = return_iter

        def return_import(module_name):
            """
            This will return a adhoc fakeprovider module if necessary,
            or fallback to the normal return of importlib.import_module.
            """
            if module_name == 'lexicon.providers.fakeprovider':
                module = ModuleType('lexicon.providers.fakeprovider')
                setattr(module, 'Provider', Provider)
                return module
            return original_import(module_name)
        mock_import.side_effect = return_import

        yield


@pytest.fixture
def lexicon_client():
    """Return the lexicon_client"""
    return importlib.import_module('lexicon.client')


@pytest.fixture(autouse=True)
def fake_provider():
    """Activate the fake_provider mock"""
    with mock_fake_provider():
        yield


def test_unknown_provider_raises_error(lexicon_client):
    with pytest.raises(ProviderNotAvailableError):
        lexicon_client.Client({'action': 'list', 'provider_name': 'unknownprovider',
                               'domain': 'example.com', 'type': 'TXT',
                               'name': 'fake', 'content': 'fake'})


def test_missing_required_client_config_parameter_raises_error(lexicon_client):
    with pytest.raises(AttributeError):
        lexicon_client.Client({'no-action': 'list', 'provider_name': 'fakeprovider',
                               'domain': 'example.com', 'type': 'TXT',
                               'name': 'fake', 'content': 'fake'})
    with pytest.raises(AttributeError):
        lexicon_client.Client({'action': 'list', 'no-provider_name': 'fakeprovider',
                               'domain': 'example.com', 'type': 'TXT',
                               'name': 'fake', 'content': 'fake'})
    with pytest.raises(AttributeError):
        lexicon_client.Client({'action': 'list', 'provider_name': 'fakeprovider',
                               'no-domain': 'example.com', 'type': 'TXT',
                               'name': 'fake', 'content': 'fake'})
    with pytest.raises(AttributeError):
        lexicon_client.Client({'action': 'list', 'provider_name': 'fakeprovider',
                               'domain': 'example.com', 'no-type': 'TXT',
                               'name': 'fake', 'content': 'fake'})


def test_missing_optional_client_config_parameter_does_not_raise_error(lexicon_client):
    lexicon_client.Client({'action': 'list', 'provider_name': 'fakeprovider',
                           'domain': 'example.com', 'type': 'TXT',
                           'no-name': 'fake', 'no-content': 'fake'})


def test_list_action_is_correctly_handled_by_provider(capsys, lexicon_client):
    client = lexicon_client.Client({'action': 'list', 'provider_name': 'fakeprovider',
                                    'domain': 'example.com', 'type': 'TXT',
                                    'name': 'fake', 'content': 'fake-content'})
    results = client.execute()

    out, _ = capsys.readouterr()

    assert 'Authenticate action' in out
    assert results['action'] == 'list'
    assert results['domain'] == 'example.com'
    assert results['type'] == 'TXT'
    assert results['name'] == 'fake'
    assert results['content'] == 'fake-content'


def test_create_action_is_correctly_handled_by_provider(capsys, lexicon_client):
    client = lexicon_client.Client({'action': 'create', 'provider_name': 'fakeprovider',
                                    'domain': 'example.com', 'type': 'TXT',
                                    'name': 'fake', 'content': 'fake-content'})
    results = client.execute()

    out, _ = capsys.readouterr()

    assert 'Authenticate action' in out
    assert results['action'] == 'create'
    assert results['domain'] == 'example.com'
    assert results['type'] == 'TXT'
    assert results['name'] == 'fake'
    assert results['content'] == 'fake-content'


def test_update_action_is_correctly_handled_by_provider(capsys, lexicon_client):
    client = lexicon_client.Client({'action': 'update', 'provider_name': 'fakeprovider',
                                    'domain': 'example.com', 'identifier': 'fake-id',
                                    'type': 'TXT', 'name': 'fake', 'content': 'fake-content'})
    results = client.execute()

    out, _ = capsys.readouterr()

    assert 'Authenticate action' in out
    assert results['action'] == 'update'
    assert results['domain'] == 'example.com'
    assert results['identifier'] == 'fake-id'
    assert results['type'] == 'TXT'
    assert results['name'] == 'fake'
    assert results['content'] == 'fake-content'


def test_delete_action_is_correctly_handled_by_provider(capsys, lexicon_client):
    client = lexicon_client.Client({'action': 'delete', 'provider_name': 'fakeprovider',
                                    'domain': 'example.com', 'identifier': 'fake-id',
                                    'type': 'TXT', 'name': 'fake', 'content': 'fake-content'})
    results = client.execute()

    out, _ = capsys.readouterr()

    assert 'Authenticate action' in out
    assert results['action'] == 'delete'
    assert results['domain'] == 'example.com'
    assert results['identifier'] == 'fake-id'
    assert results['type'] == 'TXT'
    assert results['name'] == 'fake'
    assert results['content'] == 'fake-content'



"""Integration tests for Aurora"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from define_tests.TheTests
class AuroraProviderTests(TestCase, IntegrationTests):
    """TestCase for Aurora"""
    provider_name = 'aurora'
    domain = 'example.nl'

    def _filter_headers(self):
        return ['Authorization']



"""
Integration tests for EUserv provider

Author: Matthias Schoettle (@mschoettle), 2019
"""
import re
import json

from unittest import TestCase
from lexicon.tests.providers.integration_tests import IntegrationTests

# EUserv has a limit of 10 TXT entries. Therefore, the live recordings were
# and entries removed manually during the execution of the tests.
# Otherwise, a test case will fail with a corresponding error (maximum entries reached).

# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class EUservProviderTests(TestCase, IntegrationTests):
    """Tests for EUserv"""
    provider_name = 'euserv'
    domain = 'schoettle.it'
    product_id_domain = 1

    def _filter_headers(self):
        return ['set-cookie']

    def _filter_query_parameters(self):
        return [
            ('password', 'PASSWORD'),
            ('email', 'EMAIL'),
            ('sess_id', 'SESSION_ID'),
            ('ord_no', 'ORDER_ID')]

    def _filter_response(self, response):
        for cookie in ['set-cookie', 'Set-Cookie']:
            if cookie in response['headers']:
                del response['headers'][cookie]

        if 'string' in response['body']:
            # Replace session and order id with placeholders
            response['body']['string'] = re.sub(
                br'"sess_id":{"value":"[\w.-]+"', b'"sess_id":{"value":"SESSION_ID"',
                response['body']['string'])

            response['body']['string'] = re.sub(
                br'"ord_no":{"value":"[\w.-]+"', b'"ord_no":{"value":"ORDER_ID"',
                response['body']['string'])

            # Replace orders in body with mock data (contains the minimal required data)
            filtered_body = json.loads(response['body']['string'].decode('UTF-8'))

            if 'result' in filtered_body and 'orders' in filtered_body['result']:
                filtered_body['result']['orders'] = [
                    {
                        'ord_no': {'value': 'ORDER_ID'},
                        'pg_id': {'value': self.product_id_domain},
                        'ord_description': {'value': 'Contract Name\n' + self.domain},
                    },
                ]

            response['body']['string'] = json.dumps(filtered_body).encode('UTF-8')

        return response



"""
Some small info about running live tests.

NFSN doesn't have trial accounts, so these tests can only
be run by those with an NFSN account. NFSN also requires
you to have an API key. More info here:

https://members.nearlyfreespeech.net/wiki/API/Introduction

You'll need an account to access that page.

Therefore, the following
parameters must be provided:

- LEXICON_NFSN_USERNAME -> Your NFSN username
- LEXICON_NFSN_TOKEN -> Your API Key
- LEXICON_NFSN_DOMAIN -> Domain you want to test with
"""
from unittest import TestCase

from lexicon.tests.providers.integration_tests import IntegrationTests


# Hook into testing framework by inheriting unittest.TestCase and reuse
# the tests which *each and every* implementation of the interface must
# pass, by inheritance from integration_tests.IntegrationTests
class NFSNProviderTests(TestCase, IntegrationTests):
    """TestCase for NFSN"""
    provider_name = 'nfsn'
    domain = 'koupia.xyz'

    def _filter_headers(self):
        return ['X-NFSN-Authentication']
